#!/usr/bin/env python

import glob, os, sys

# The next statements assure importation of qwt and qwtc from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../qwt/.libs/libqwtcmodule.*')
if globs:                               # found PyQwt modules made by libtool
    sys.path.insert(0, '../qwt')
    sys.path.insert(0, '../qwt/.libs')
globs = glob.glob('../build/lib.*/libqwtcmodule.*')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *
from LinearAlgebra import *

# realistic
ch = 1.28e-12
cv = 21.175e-12
lv = 10.000e-9
lh =  3.000e-9
lq = 0.002

def g_c(value, omega):
    return 1j*omega*value

def g_l(value, omega):
    return 1.0/((lq+1j)*omega*value)

def g_r(value, omega):
    return 1.0/value

def rhs(omega):
    return 2*sqrt((1-omega*omega*cv*lv)/(lh*cv))

# input node = -1
# ground node = -2

def zMatrix(circuit, omega):
    # find the number of modes and the node connected to the "input" branch
    highs = []
    for func, lo, hi, value in circuit:
        highs.append(hi)
        if lo == -1:
            inNode = hi
    nodes = 1 + max(highs)
    # the hard work
    z = zeros(len(omega), Complex)
    for i in range(len(omega)):
        a = zeros((nodes, nodes), Complex)  # nodal conductance matrix
        b = zeros(nodes, Complex)           # source current vector
        for func, lo, hi, value in circuit:
            g = func(value, omega[i])
            if lo == -2:                    # ground
                a[hi][hi] += g
            elif lo == -1:                  # input
                b[hi] += g
                a[hi][hi] += g
            else:
                a[lo][lo] += g
                a[hi][hi] += g
                a[lo][hi] -= g
                a[hi][lo] -= g
        v = solve_linear_equations(a, b)
        z[i] = 1.0/(b[inNode]*(1-v[inNode]))
    return z


class SmithChart(QWidget):
    def __init__(self,*args):
        apply(QWidget.__init__,(self,) + args)
	
	# The QwtPlot widget
	self.griPlot = QwtPlot("Smith Chart", self)
        self.gnfPlot = QwtPlot("abs(Gamma) - f", self)
        self.zPlot = QwtPlot("re(Z), im(Z) - f", self)
        self.rPlot = QwtPlot("Intersect Plot", self)
        # FIXME : look in .sip for the last option (step): maybe double = 0.0
        self.griPlot.setAxisScale(QwtPlot.xBottom, -1.0, 1.0)
        self.griPlot.setAxisScale(QwtPlot.yLeft, -1.0, 1.0)
	# self.zPlot.setAxisScale(QwtPlot.xBottom, 300, 400)
        
        # circuit description
        circuit = [
            (g_c, -1,  1, ch),
            (g_c,  0,  1, cv),
            (g_l, -2,  1, lv),
            (g_l,  0,  2, lh),
            (g_c,  2,  3, cv),
            (g_l, -2,  3, lv),
            (g_l,  2,  4, lh),
            (g_c,  4,  5, cv),
            (g_l, -2,  5, lv),
            (g_l,  4,  6, lh),
            (g_c,  6,  7, cv),
            (g_l, -2,  7, lv),
            (g_l,  6,  8, lh),
            (g_c,  8,  9, cv),
            (g_l, -2,  9, lv),
            (g_l,  8, 10, lh),
            (g_c, 10, 11, cv),
            (g_l, -2, 11, lv),
            (g_l, 10,  0, lh),
            ]

        fmax = 1.0/(2*pi*sqrt(lv*cv))
        fs = 2*pi*arrayrange(3*fmax/4, fmax, 50e3)
        zs = zMatrix(circuit, fs)
        gs = (zs-50.0)/(zs+50.0)
        sect = rhs(fs)/(2e6*pi)
        fs = fs/(2e6*pi)

        curveSnail = self.griPlot.insertCurve("im(Gamma) vs re(Gamma)")
        self.griPlot.setCurveData(curveSnail, gs.real, gs.imag)
        curveAbsG = self.gnfPlot.insertCurve("abs(Gamma) vs f")
        self.gnfPlot.setCurveData(curveAbsG, fs, abs(gs))
        self.gnfPlot.setAxisScale(QwtPlot.xBottom, 300.0, 310.0)
        curveReLadder = self.zPlot.insertCurve("re(zLadder)")
        curveImLadder = self.zPlot.insertCurve("im(zLadder)")
        self.zPlot.setCurveData(curveReLadder, fs, zs.real)
        self.zPlot.setCurvePen(curveReLadder, QPen(Qt.red))
        self.zPlot.setCurveData(curveImLadder, fs, zs.imag)
        self.zPlot.setCurvePen(curveImLadder, QPen(Qt.blue))
        curveSect1 = self.rPlot.insertCurve("mode 1")
        curveSect2 = self.rPlot.insertCurve("mode 2")
        curveSect3 = self.rPlot.insertCurve("mode 3")
        curveSectR = self.rPlot.insertCurve("intersect")
        self.rPlot.setCurveData(curveSectR, fs, fs)
        self.rPlot.setCurvePen(curveSectR, QPen(Qt.red))
        self.rPlot.setCurveData(curveSect1, fs, sin(pi/6)*sect)
        self.rPlot.setCurvePen(curveSect1, QPen(Qt.green))
        self.rPlot.setCurveData(curveSect2, fs, sin(pi/3)*sect)
        self.rPlot.setCurvePen(curveSect2, QPen(Qt.green))
        self.rPlot.setCurveData(curveSect3, fs, sin(pi/2)*sect)
        self.rPlot.setCurvePen(curveSect3, QPen(Qt.green))

    def resizeEvent(self, e):
        x = e.size().width()/2
        y = e.size().height()/2
    	self.griPlot.resize(x, y)
	self.griPlot.move(0, 0)
    	self.gnfPlot.resize(x, y)
	self.gnfPlot.move(0, y)
        self.zPlot.resize(x, y)
        self.zPlot.move(x, 0)
        self.rPlot.resize(x, y)
        self.rPlot.move(x, y)
	

if __name__ == '__main__':
    app = QApplication(sys.argv)
    chart = SmithChart()
    app.setMainWidget(chart)
    chart.resize(700, 700)
    chart.show()
    app.exec_loop()
