#!/usr/bin/env python

import os, re
from os.path import exists, join
from glob import glob
from types import *
import pre_sip

from distutils.ccompiler import new_compiler
from distutils.command.build_ext import build_ext
from distutils.command.install_data import install_data
from distutils.core import Extension
from distutils.spawn import find_executable
from distutils.sysconfig import get_config_vars, get_python_inc, get_python_lib
from distutils.util import byte_compile


class PyQtExtension(Extension):

    def __init__(self, name, sources,
                 sip_spec_dirs=None,    # PyQtExtension
                 sip_fix=None,          # PyQtExtension
                 moc_headers=None,      # PyQtExtension
                 include_dirs=None,
                 define_macros=None,
                 undef_macros=None,
                 library_dirs=None,
                 libraries=None,
                 runtime_library_dirs=None,
                 extra_objects=None,
                 extra_compile_args=None,
                 extra_link_args=None):

        assert isinstance(name, StringType), "'name' must be a string"
        assert (sources == []) or (isinstance(sources, ListType) and
                map(isinstance, sources, [StringType]*len(sources))), \
                "'sources' must be an empty list or a list of strings"
        # FIXME: assert three arguments?
        assert sip_fix == None or isinstance(sip_fix, FunctionType), \
               ("'sip_fix' must be a function of 'fix(version, dir, name)',\n"
                "/t'version' is obtained by 'sip -V'\n"
                "/t'dir' is the directory of sip-generated source code'\n"
                "/t'name' is the name of the extension'")

        self.sip_spec_dirs = sip_spec_dirs or []
        self.moc_headers = moc_headers or []

        self.sip_fix = sip_fix

        self.last = name
        self.sip_file = join('sip', '%s.sip' % self.last)
        self.sip_wrapper = join(self.last, '%s.py' % self.last)

        # check if all your hand written C++ sources exist
        for source in sources:
            if not exists(source):
                raise DistutilsFileError, "file '%s' does not exist" % source 
        sources = sources
        # the main sip-generated source file, find the rest after running sip
        sources.append(join(self.last, '%scmodule.cpp' % self.last))
        # handle the header files to be processed by moc; giving more sources
        self.moc_iofiles = []
        for header in self.moc_headers:
            if not exists(source):
                raise DistutilsFileError, "file '%s' does not exist" % header 
            head, tail = os.path.split(header)
            (root, _) = os.path.splitext(tail)
            target = join(head, 'moc_%s.cpp' % root)
            self.moc_iofiles.append((header, target))
            sources.append(target)

        if os.name == 'nt':
            self.name = 'lib%sc' % self.last
        else:
            self.name = 'lib%scmodule' % self.last
        # now call the base class constructor with the sources we've found
        Extension.__init__(self,
                           self.name,
                           sources,
                           include_dirs,
                           define_macros,
                           undef_macros,
                           library_dirs,
                           libraries,
                           runtime_library_dirs,
                           extra_objects,
                           extra_compile_args,
                           extra_link_args)
        self.include_dirs.append(join(os.getcwd(), self.last))


# hack, hack, transforms the UnixCCompiler into a UnixC++Compiler
def customize_compiler(compiler):
    """Hide distutils.sysconfig.customize(compiler).
    """
    if compiler.compiler_type == "unix":
        # ./configure --with-cxx=c++ sets CXX=c++ in config/Makefile
        (cc, cxx, opt, ccshared, ldshared, so_ext) = \
            get_config_vars('CC', 'CXX', 'OPT', 'CCSHARED', 'LDSHARED', 'SO')
        # fall back on g++
        cxx = cxx or 'g++'
        if not find_executable(cxx):
            raise SystemExit, "Fails to find C++ compiler '%s' in path" % cxx
        cxx_cmd = cxx + ' -Wall ' + opt + ' -fno-exceptions'
        compiler.set_executables(preprocessor=cxx + " -E",
                                 compiler=cxx_cmd,
                                 compiler_so=cxx_cmd + ' ' + ccshared,
                                 linker_so=ldshared.replace(cc, cxx),
                                 linker_exe=cxx)
        compiler.shared_lib_extension = so_ext
    if compiler.compiler_type == "msvc":
        compiler.compile_options = [ '/nologo', '/W3', '/O1' ]
        compiler.ldflags_shared = [ '/DLL', '/nologo', '/SUBSYSTEM:windows' ] 

def timeline_options(sip_version, qt_version):
    message = "PyQt is not compatible with QT_VERSION '%s'"
    if (sip_version == '2.5'):
        if qt_version < 143:
            raise SystemExit, message % qt_version 
        if qt_version == 143:
            return '-t Qt_143'
        if qt_version < 200:
            raise SystemExit, message % qt_version
        if qt_version < 210:
            return '-t Qt_2_00'
        if qt_version < 220:
            return '-t Qt_2_1_0'
        if qt_version < 230:
            return '-t Qt_2_2_0'
        if qt_version == 230:
            return '-t Qt_2_3_0'
        if qt_version == 231:
            return '-t Qt_2_3_1'
        if qt_version == 300:
            return '-t Qt_3_0_0'
        else: # FIXME: Qt-4.0.0
            raise SystemExit, message % qt_version
    else: # FIXME: sip-2.6
        return ''


class build_ext_pyqt(build_ext):
    description = "build 'PyQt' extensions, using 'sip'"

    user_options = [
        ('qt-dir=', None,
         "Qt's root directory, e.g. /usr/lib/qt2"),
        ('qt-bindir=', None,
         "Qt's binary directory for moc & friends, e.g. /usr/lib/qt2/bin"),
        ('qt-incdir', None,
         "Qt's include directory, e.g. /usr/lib/qt2/include"),
        ('moc-program=', None,
         "full path of moc"),
        ('sip-program=', None,
         "full path of sip"),
        ('sip-incdir=', None,
         "sip's include directory for sip.h & friends, e.g. /usr/include/sip"),
        ('sip-only', None,
         "run sip to build Python and C++ sources from *.sip files")
        ] + build_ext.user_options[:12] # shielding the SWIG related options

    def initialize_options(self):
        self.qt_dir=None
        self.qt_bindir=None
        self.qt_incdir=None
        self.qt_libdir=None
        self.moc_program=None
        self.sip_program=None
        self.sip_incdir=None
        self.sip_only=0
        build_ext.initialize_options(self)

    def finalize_options(self):
        build_ext.finalize_options(self)

        # try to get our options
        if not self.qt_dir:
            if os.environ.has_key("QTDIR"):
                self.qt_dir = os.environ["QTDIR"]
        if self.qt_dir:
            if not self.qt_bindir:
                self.qt_bindir = join(self.qt_dir, 'bin')
            if not self.qt_incdir:
                self.qt_incdir = join(self.qt_dir, 'include')
            if not self.qt_libdir:
                self.qt_libdir = join(self.qt_dir, 'lib')
            if not self.moc_program:
                self.moc_program = find_executable('moc', self.qt_bindir)

        if not self.moc_program:
            self.moc_program = find_executable('moc')
        if not self.sip_program:
            self.sip_program = find_executable('sip')

        # BOMB out, if self.qt_incdir is WRONG, get QT_VERSION from 'qglobal.h'
        if not self.qt_incdir:
            raise SystemExit, ("Edit 'setup.py': 'qt_incdir=%s' fails." %
                               self.qt_incdir)
        else:   
            filename = join(self.qt_incdir, 'qglobal.h')
            try:
                file = open(filename)
                text = file.read()
                file.close()
                mo = re.compile(
                    r'^#define\s+QT_VERSION\s+(\d\d\d)', re.M).search(text, 1)
                if mo:
                    self.qt_version = int(mo.group(1))
                else:
                    raise IndexError
            except IOError:
                raise SystemExit, \
                      "Edit 'setup.py': %s does not exist." % filename
            except IndexError:
                raise SystemExit, \
                      "Edit 'setup.py': %s does not '#define QT_VERSION'." % \
                      filename

        # BOMB out, if self.sip_program is not OK
        if not self.sip_program:
            raise SystemExit, ("Edit 'setup.py': 'sip_program=%s' fails." %
                               self.sip_program)
        self.sip_version = os.popen('%s -V' % self.sip_program).read().rstrip()
        if self.sip_version == 'unknown': # FIXME: sip sucks on Windows
            self.announce("'sip -V' returns 'unknown': '2.5' on Windows?")
            self.sip_version = '2.5'
        if not (self.sip_version == '2.3' or
                self.sip_version == '2.4' or
                self.sip_version == '2.5'):
            raise SystemExit, \
                  'sip-2.3, -2.4, -2.5 required to build PyQwt'

        # finalize
        self.sip_timeline = timeline_options(self.sip_version, self.qt_version)
        if self.qt_incdir:
            self.include_dirs.append(self.qt_incdir)
        if self.sip_incdir:
            self.include_dirs.append(self.sip_incdir)
        if self.qt_libdir:
            self.library_dirs.append(self.qt_libdir) 
        if os.name == 'nt':
            qt_lib = 'qt%s' % self.qt_version
            extra_version = '-Qt-%s' % self.qt_version
            self.define = self.define or []
            self.define.append(('SIP_USE_DLL', None))
            if not glob(join(self.qt_libdir, qt_lib + '.lib')):
                qt_lib = 'qt-mt230nc' # fall back on NonCommercial
                extra_version = '-Qt-NC'
                if not glob(join(self.qt_libdir, qt_lib + '.lib')):
                    raise SystemExit, \
                          'Qt-library does not match with Qt-version'
                self.define.append(('QT_DLL', None))
            # hack, hack, ...
            self.distribution.metadata.version = \
                 self.distribution.metadata.version + extra_version 
            self.libraries = self.libraries + ['sip', qt_lib]
        else:
            self.libraries = self.libraries + ['sip', 'qt']

    def run_sip(self, extension, name):
        # run sip with appropriate arguments
        includes = ['-I', 'sip']
        for dir in extension.sip_spec_dirs:
            includes.append('-I')
            # hack C code does not understand Windows path separators
            includes.append(dir.replace('\\', '/'))
        cmd = [self.sip_program] + includes + ['-c', name] + \
              self.sip_timeline.split() + \
              [extension.sip_file.replace('\\', '/')]
        self.spawn(cmd)
        # look if we have a function to postprocess the sip output
        if extension.sip_fix:
            extension.sip_fix(self.sip_version, name, name)
        # keep track of of the timeline
        file = open(join(name, 'code'), 'w')
        file.write('%s\n' % self.sip_timeline)
        file.close()
        
    def run_sip_helper(self, name):
        # fake building and running sip_helper
        lines = ['#undef SIP_VERS_WS_WIN\n']
        if os.name == 'nt':
            lines.append('#define SIP_VERS_WS_WIN\n')
        lines.append('#undef SIP_VERS_WS_X11\n')
        if os.name != 'nt':
            lines.append('#define SIP_VERS_WS_X11\n')
        # more work for sip-2.4
        if self.sip_version < '2.5':
            lines.append('#undef SIP_VERS_Qt_2_3_0\n')
            if self.qt_version >= 230:
                lines.append('#define SIP_VERS_Qt_2_3_0\n')
            lines.append('#undef SIP_VERS_Qt_2_2_0\n')
            if self.qt_version >= 220:
                lines.append('#define SIP_VERS_Qt_2_2_0\n')
            lines.append('#undef SIP_VERS_Qt_2_1_0\n')
            if self.qt_version >= 210:
                lines.append('#define SIP_VERS_Qt_2_1_0\n')
            lines.append('#undef SIP_VERS_Qt_2_00\n')
            if self.qt_version >= 200:
                lines.append('#define SIP_VERS_Qt_2_00\n')
            lines.append('#undef SIP_VERS_Qt_1_43\n')
            if self.qt_version >= 143:
                lines.append('#define SIP_VERS_Qt_1_43\n')
        file = open(join(name, 'sip%sVersion.h' % name), 'w')
        file.writelines(lines)
        file.close()

    def run_moc(self, extension, name):
        # run moc, we get along without the preprocessor step!
        sipProxy_h = join(name, 'sip%sProxy%s.h' % (name, name))
        sipProxy_moc = join(name, 'sip%sProxy%s.moc' % (name, name))
        self.make_file([sipProxy_h], sipProxy_moc, self.spawn,
                       ([self.moc_program, sipProxy_h, '-o', sipProxy_moc],))
        self.announce('"%s" has made "%s"' % (self.moc_program, sipProxy_moc))
        for ifile, ofile in extension.moc_iofiles:
            self.make_file([ifile], ofile, self.spawn,
                           ([self.moc_program, ifile, '-o', ofile],))


    def run(self):
        # BUG: if 'qt', it needs to be the first extension
        for extension in self.extensions:
            assert isinstance(extension, PyQtExtension)
            name = extension.last
            # generate the appropiate sip interface files
            pre_sip.pre_sip(self.sip_version, 'sip')
            # sip sources are 'name.sip' and files included by 'name.sip'
            extension.sip_files = [ extension.sip_file ]
            file = open(extension.sip_file, 'r')
            IncludeRE = re.compile(r'^%Include (.*\.sip)')
            for line in file.readlines(): 
                m = re.match(IncludeRE, line)
                if m:
                    inclusion = join('sip', m.group(1))
                    if exists(inclusion):
                        extension.sip_files.append(inclusion)
                    else:
                        raise DistutilsFileError, \
                              "file '%s' does not exist" % source
            file.close()
            # for sip >= 2.5 we need to check if the generated code is OK
            # if not OK, we force rebuild_src by removing 'name/name.py
            if self.sip_version >= '2.5':
                code = join(name, 'code')
                if not (exists(code) and
                        self.sip_timeline == open(code).read().rstrip()):
                    if exists(extension.sip_wrapper):
                        os.remove(extension.sip_wrapper)
            # run sip and post-process, if a sip file post-dates 'name/name.py'
            self.make_file(extension.sip_files,
                           extension.sip_wrapper,
                           self.run_sip, (extension, name))
            # FUDGE: append the sip-generated *.cpp to the extension.sources
            sources = glob(join(name, "sip%s*.cpp" % name))
            for source in sources:
                extension.sources.append(source)
            # finish
            self.run_sip_helper(name)
            self.run_moc(extension, name)

        if self.sip_only:
            return 1

        self.compiler = new_compiler(verbose=self.verbose,
                                     dry_run=self.dry_run,
                                     force=self.force)
        customize_compiler(self.compiler)
            
        if self.include_dirs is not None:
            self.compiler.set_include_dirs(self.include_dirs)
        if self.define is not None:
            # 'define' option is a list of (name,value) tuples
            for (name,value) in self.define:
                self.compiler.define_macro(name, value)
        if self.undef is not None:
            for macro in self.undef:
                self.compiler.undefine_macro(macro)
        if self.libraries is not None:
            self.compiler.set_libraries(self.libraries)
        if self.library_dirs is not None:
            self.compiler.set_library_dirs(self.library_dirs)
        if self.rpath is not None:
            self.compiler.set_runtime_library_dirs(self.rpath)
        if self.link_objects is not None:
            self.compiler.set_link_objects(self.link_objects)
        self.compiler.set_include_dirs(self.include_dirs)
        self.compiler.set_libraries(self.libraries)

#        self.compiler.add_library_dir(get_python_lib())
#        self.compiler.add_library_dir(os.path.abspath(self.build_lib))

        self.build_extensions()

        for extension in self.extensions:
            name = extension.last
            prefix = self.build_lib
            if prefix[-1] != os.sep:
                prefix = prefix + os.sep
            wrapper = join(name, '%s.py' % name)
            self.copy_file(wrapper, self.build_lib)
            source = join(self.build_lib, os.path.basename(wrapper))
            byte_compile([source],
                         optimize=0,
                         force=self.force,
                         prefix=prefix,
                         verbose=self.verbose,
                         dry_run=self.dry_run)
#            if self.optimize > 0:
#                byte_compile([pyfile],
#                             optimize=self.optimize,
#                             force=self.force,
#                             prefix=prefix,
#                             verbose=self.verbose,
#                             dry_run=self.dry_run)

# sets 'install_dir' to 'install_lib' instead of 'install_data'
class install_data_pyqt(install_data):
    def finalize_options (self):
        self.set_undefined_options('install',
                                   ('install_lib', 'install_dir'),
                                   ('root', 'root'),
                                   ('force', 'force'))

