/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_counter.h"
#include "../include/qwt_math.h"
#include "../include/qwt_arrbtn.h"

#include <qlined.h>
#include <qlabel.h>

static const int MAXBTNS=3;
static const int MIN_LBL_WIDTH = 50;

/*!
  \brief ctor

  The default number of buttons is set to 2. The default increments are:
  \li Button 1: 1 step
  \li Button 2: 10 steps
  \li Button 3: 100 steps
  
  \param parent
  \param name Forwarded to QWidget's ctor.
 */
QwtCounter::QwtCounter(QWidget *parent, char *name )
: QWidget(parent,name)
{
    int i;
    
    d_nButtons = 2;
    d_Inc[0] = 1;
    d_Inc[1] = 10;
    d_Inc[2] = 100;
    
    d_tmrIntv = 200;

    lblValue = new QLabel(this);
    lblValue->setNum(0.0);
    lblValue->setAlignment(AlignCenter);
    lblValue->setFrameStyle(QFrame::Panel|QFrame::Sunken);

    lblValue->setMargin(1);           // for sizeHint()

    
    for(i=0;i<MAXBTNS;i++)
    {
	btnL[i] = new QwtArrowButton(i+1, QwtArrowButton::Down,this);
	btnR[i] = new QwtArrowButton(i+1, QwtArrowButton::Up, this);
    }

    connect(btnL[0], SIGNAL(pressed()), SLOT(btnL1Dn()));
    connect(btnL[1], SIGNAL(pressed()), SLOT(btnL2Dn()));
    connect(btnL[2], SIGNAL(pressed()), SLOT(btnL3Dn()));
    connect(btnR[0], SIGNAL(pressed()), SLOT(btnR1Dn()));
    connect(btnR[1], SIGNAL(pressed()), SLOT(btnR2Dn()));
    connect(btnR[2], SIGNAL(pressed()), SLOT(btnR3Dn()));
    connect(btnL[0], SIGNAL(released()), SLOT(btnUp()));
    connect(btnL[1], SIGNAL(released()), SLOT(btnUp()));
    connect(btnL[2], SIGNAL(released()), SLOT(btnUp()));
    connect(btnR[0], SIGNAL(released()), SLOT(btnUp()));
    connect(btnR[1], SIGNAL(released()), SLOT(btnUp()));
    connect(btnR[2], SIGNAL(released()), SLOT(btnUp()));
    
    setRange(0.0,1.0,0.001);
    setValue(0.0);
    
}

//! dtor
QwtCounter::~QwtCounter()
{
    delete btnL[0];
    delete btnL[1];
    
    delete btnR[0];
    delete btnR[1];
    
    delete lblValue;
}

/*!
  Specify the number of steps by which the value
  is incremented or decremented when a specified button
  is pushed.
  
  \param btn One of \c QwtCounter::Button1, \c QwtCounter::Button2,
             \c QwtCounter::Button3
  \param nSteps	Number of steps
*/
void QwtCounter::setIncSteps(QwtCounter::Button btn, int nSteps)
{
    if (( btn >= 0) && (btn < ButtonCnt)) 
       d_Inc[btn] = nSteps;
}

/*!
  \return the number of steps by which a specified button increments the value
  or 0 if the button is invalid.
  \param btn One of \c QwtCounter::Button1, \c QwtCounter::Button2,
  \c QwtCounter::Button3
*/
int QwtCounter::incSteps(QwtCounter::Button btn) const
{
    if (( btn >= 0) && (btn < ButtonCnt)) 
       return d_Inc[btn];
    else
       return 0;
}


/*!
  \brief Notify a change of value
*/
void QwtCounter::valueChange()
{
    lblValue->setNum(value());
    emit valueChanged(value());
}


/*!
  \brief Specify the number of buttons on each side of the label
  \param n Number of buttons
*/
void QwtCounter::setNumButtons(int n)
{

    if ((n > 0) && (n <= ButtonCnt))
       d_nButtons = n;
    resize(size());
}

//!  Button 1 Left Down
void QwtCounter::btnL1Dn()
{
    if (tmrID) killTimer(tmrID);
    d_tick = 0;
    d_incValue = -d_Inc[0];
    tmrID = startTimer(d_tmrIntv);
}

//!  Button 2 Left Down
void QwtCounter::btnL2Dn()
{
    if (tmrID) killTimer(tmrID);
    d_tick = 0;
    d_incValue = -d_Inc[1];
    tmrID = startTimer(d_tmrIntv);
}

//!  Button 3 Left Down
void QwtCounter::btnL3Dn()
{
    if (tmrID) killTimer(tmrID);
    d_tick = 0;
    d_incValue = -d_Inc[2];
    tmrID = startTimer(d_tmrIntv);
}


//!  Button 1 Right Down
void QwtCounter::btnR1Dn()
{
    if (tmrID) killTimer(tmrID);
    d_tick = 0;
    d_incValue = d_Inc[0];
    tmrID = startTimer(d_tmrIntv);
}

//!  Button 2 Right Down
void QwtCounter::btnR2Dn()
{
    if (tmrID) killTimer(tmrID);
    d_tick = 0;
    d_incValue = d_Inc[1];
    tmrID = startTimer(d_tmrIntv);
}

//!  Button 3 Right Down
void QwtCounter::btnR3Dn()
{
    if (tmrID) killTimer(tmrID);
    d_tick = 0;
    d_incValue = d_Inc[2];
    tmrID = startTimer(d_tmrIntv);
}

//!  Button Released
void QwtCounter::btnUp()
{
    if (!d_tick)
    {
	incValue(d_incValue);
	d_tick = 0;
    }
    if (tmrID) killTimer(tmrID);
    emit buttonReleased(value());
}


/*!
  \brief Resize the counter
  \param e Todo...
*/
void QwtCounter::resizeEvent(QResizeEvent *e)
{
    int btnWidth,i;
    
    QSize r(e->size());
    btnWidth = r.height();
    
    btnWidth = qwtMin(r.height(), (r.width() - MIN_LBL_WIDTH) / ( 2 * d_nButtons));
    btnWidth = qwtMax(btnWidth,5);
    
    for (i=0;i<d_nButtons;i++)
    {
	btnL[i]->resize(btnWidth,r.height());
	btnL[i]->move((d_nButtons - i - 1) * btnWidth, 0);
	btnR[i]->resize(btnWidth, r.height());
	btnR[i]->move(r.width() - (d_nButtons - i) * btnWidth, 0);
	btnL[i]->show();
	btnR[i]->show();
    }
    
    for (i=d_nButtons; i<MAXBTNS; i++)
    {
	btnL[i]->hide();
	btnR[i]->hide();
    }

    lblValue->resize(r.width() - 2*d_nButtons * btnWidth, r.height());
    lblValue->move(d_nButtons * btnWidth, 0);

}


/*!
  \brief Timer event handler

  A timer event occurs when a mouse is pressed
  and hold down on one of the buttons.
  The default implementation increments the
  counter's value.
  \param e Todo...
*/
void QwtCounter::timerEvent(QTimerEvent *e)
{
    d_tick = 1;
    incValue(d_incValue);
}

/*!
  \brief Notify change of font

  This function updates the fonts of all widgets
  contained in QwtCounter.
  \param f new font
*/
void QwtCounter::fontChange(const QFont& f)
{
    int i;
    lblValue->setFont(font());
    for (i = 0;i <ButtonCnt; i++)
    {
	btnL[i]->setFont(font());
	btnR[i]->setFont(font());
    } 
}

/*!
  \return A recommended size for the counter.
*/
QSize QwtCounter::sizeHint() const
{
    QSize rv;
    int h =  qwtMax(lblValue->sizeHint().height(), btnL[0]->sizeHint().height());
    rv.setHeight(h);
    rv.setWidth( qwtMax(lblValue->sizeHint().width(), MIN_LBL_WIDTH) + d_nButtons * h * 2); 
    return rv;
}

/*!
  \brief Activates/deactivates user input
  \param tf enabled/disabled
*/
void QwtCounter::setEnabled(bool tf)
{
    int i;

    QWidget::setEnabled(tf);
    
    for (i=0;i<ButtonCnt;i++)
    {
	btnL[i]->setEnabled(tf);
	btnR[i]->setEnabled(tf);
    }

    lblValue->setEnabled(tf);
    
}
