/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_marker.h"
#include "../include/qwt_math.h"
#include <qpainter.h>

static const int LabelDist = 2;

//! Ctor
QwtMarker::QwtMarker()
{
    d_align = Qt::AlignCenter;
    d_style = NoLine;
}

//! Destructor
QwtMarker::~QwtMarker()
{
    
}

/*!
  \brief Draw the marker
  \param p Painter
  \param x X position
  \param y Y position
  \param r Bounding rectangle
*/
void QwtMarker::draw(QPainter *p, int x, int y, const QRect &r)
{
    QRect tr;
    QSize sSym;
    int tw, th, lw, lw1;
    
    
    // draw lines
    if (d_style != NoLine)
    {
	p->setPen(d_pen);
	if ((d_style == HLine) || (d_style == Cross))
	   p->drawLine(r.left(), y, r.right(), y);
	if ((d_style == VLine)||(d_style == Cross))
	   p->drawLine(x, r.top(), x, r.bottom());
    }

    // draw symbol
    if (d_sym.style() != QwtSymbol::None)
    {
	sSym = d_sym.size();
	d_sym.draw(p, x - (sSym.width() - 1) / 2 ,y - (sSym.width() - 1) / 2);
    }
    else
    {
	sSym.setWidth(0);
	sSym.setHeight(0);
    }
    

    // draw label
    if (d_label != "")
    {
	p->setPen(d_tPen);
	p->setFont(d_font);
	th = p->fontMetrics().height();
	tw = p->fontMetrics().width(d_label);
	lw = qwtMax(int(d_pen.width()), 1);

	if ((d_style == VLine) || (d_style == HLine))
	{
	    lw1 = (lw + 1) / 2 + LabelDist;
	    lw = lw / 2 + LabelDist;
	}
	else 
	{
	    lw1 = qwtMax((lw + 1) / 2, (sSym.width() + 1) / 2) + LabelDist;
	    lw = qwtMax(lw / 2, (sSym.width() + 1) / 2) + LabelDist;
	}

	
	if (d_style == VLine)
	{
	    if (d_align & (int) Qt::AlignTop)
	       tr.setY(r.top() + LabelDist);
	    else if (d_align & (int) Qt::AlignBottom)
	       tr.setY(r.bottom() - LabelDist - th);
	    else
	       tr.setY(r.top() + (r.bottom() - r.top()) / 2);
	}
	else 
	{
	    if (d_align & (int) Qt::AlignTop)
	       tr.setY(y - lw - th);
	    else if (d_align & (int) Qt::AlignBottom)
	       tr.setY(y + lw1);
	    else
	       tr.setY(y - th/2);
	}


	if (d_style == HLine)
	{
	    if (d_align & (int) Qt::AlignLeft)
	       tr.setX(r.left() + LabelDist);
	    else if (d_align & (int) Qt::AlignRight)
	       tr.setX(r.right() - tw - LabelDist);
	    else
	       tr.setX(r.left() + (r.right() - r.left()) / 2);
	}
	else
	{
	    if (d_align & (int) Qt::AlignLeft)
	       tr.setX(x - tw - lw);
	    else if (d_align & (int) Qt::AlignRight)
	       tr.setX(x + lw1);
	    else
	       tr.setX(x - tw/ 2);
	}

	tr.setHeight(th);
	tr.setWidth(tw);	

	p->drawText(tr, Qt::AlignTop|Qt::AlignHCenter, d_label); 
	
    }
    
    
}

/*!
  \brief Specify the font for the label
  \param f New font
*/
void QwtMarker::setFont(const QFont &f)
{
    d_font = f;
    markerChanged();
}



/*!
  \brief Set the line style
  \param st Line style. Can be one of QwtMarker::NoLine,
	QwtMarker::HLine, QwtMarker::VLine or QwtMarker::Cross
*/
void QwtMarker::setLineStyle(QwtMarker::LineStyle st)
{
    d_style = st;
    markerChanged();
}



/*!
  \brief Assign a symbol
  \param s New symbol 
  \sa QwtSymbol
*/
void QwtMarker::setSymbol(const QwtSymbol &s)
{
    d_sym = s;
    markerChanged();
}

/*!
  \brief Set the label
  \param txt label text	
*/
void QwtMarker::setLabel(const char *txt)
{
    d_label = txt;
    markerChanged();
}

/*!
  \brief Set the alignment of the label

  The alignment determines where the label is drawn relative to
  the marker's position.

  \param align Alignment. A combination of AlignTop, AlignBottom,
	AlignLeft, AlignRight, AlignCenter, AlgnHCenter,
	AlignVCenter.  
*/
void QwtMarker::setLabelAlignment(int align)
{
    d_align = align;
    markerChanged();
}

/*!
  \brief Specify a pen for the line.
  \param p New pen
*/
void QwtMarker::setLinePen(const QPen &p)
{
    d_pen = p;
    markerChanged();
}

/*!
  \brief Specify a pen for the label
  \param p New pen
*/
void QwtMarker::setLabelPen(const QPen &p)
{
    d_tPen = p;
    markerChanged();
}

/*!
  \brief Assignment operator
  \param m Marker
*/
const QwtMarker& QwtMarker::operator=(const QwtMarker &m)
{
    d_label = m.d_label;
    d_pen = m.d_pen;
    d_tPen = m.d_tPen;
    d_font = m.d_font;
    d_sym = m.d_sym;
    d_xMap = m.d_xMap;
    d_yMap = m.d_yMap;
    d_align = m.d_align;
    d_style = m.d_style;

    return *this;
}
