/* -*- mode: C++ ; c-file-style: "stroustrup" -*- *****************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/


#include "../include/qwt_plot.h"
#include "../include/qwt_plot_dict.h"
#include "../include/qwt_rect.h"
#include "../include/qwt_scale.h"
#include "../include/qwt_legend.h"
#include "../include/qwt_plot_pixframe.h"
#include <qlabel.h>
#include "../include/qwt_math.h"
#include <qwmatrix.h>
#include <qpaintdevicemetrics.h>
#include <qbitmap.h>
#include <qregexp.h>

const int QwtPlot::DefaultXAxis = QwtPlot::xBottom;
const int QwtPlot::DefaultYAxis = QwtPlot::yLeft;
static const int ScrBarWidth = 20;


QFont& QwtPlot::DummyFont()
{
  static QFont font;
  return font;
}

QString& QwtPlot::DummyString()
{
  static QString str("");
  return str;
}

QwtScaleDiv& QwtPlot::DummyScale()
{
  static QwtScaleDiv qsd;
  return qsd;
}

QPen& QwtPlot::DummyPen()
{
  static QPen pen(NoPen);
  return pen;
}

QwtSymbol& QwtPlot::DummySymbol()
{
  static QwtSymbol symb;
  return symb;
}


//------------------------------------------------------------
// Constructors / Destructors
//------------------------------------------------------------

//! Constructor
QwtPlot::QwtPlot(QWidget *p, const char *name)
: QFrame(p,name), d_grid(this)
{
    initPlot();
}


/*!
  \brief Constructor
  \param title Title text
  \param parent Parent widget
  \param name Widget name
 */
QwtPlot::QwtPlot(const char *title, QWidget *p, const char *name)
: QFrame(p,name), d_grid(this)
{
    initPlot(title);
}    

/*!
  \brief Initializes a QwtPlot instance
  \param title Title text
 */
void QwtPlot::initPlot(const char *title)
{

    QString txt(title);
    
    QFont fscl("Helvetica", 10);
    QFont fttl("Helvetica", 12, QFont::Bold);
    
    d_scale[yLeft] = new QwtScale(QwtScale::Left,this);
    d_scale[yRight] = new QwtScale(QwtScale::Right,this);
    d_scale[xTop] = new QwtScale(QwtScale::Top,this);
    d_scale[xBottom] = new QwtScale(QwtScale::Bottom,this);
    d_frmPlot = new QwtPlotPixFrame(this);
    d_lblTitle = new QLabel(title, this);
    d_legend = new QwtLegend(this);

    d_plotBorder = 2;
    d_titleDist = 20;
    d_titleEnabled = (!txt.isEmpty());
    d_autoReplot = FALSE;

    d_legendPos = Qwt::Bottom;
    d_legendEnabled = FALSE;
    
    initAxes();
    
    d_lblTitle->setFont(QFont("Helvetica", 14,QFont::Bold));
    d_lblTitle->setAlignment(AlignHCenter|AlignBottom);

    d_frmPlot->setFrameStyle(QFrame::Panel|QFrame::Sunken);
    d_frmPlot->setLineWidth(2);
    d_frmPlot->setMidLineWidth(0);

    d_margin = 10;
    setLineWidth(2);
    setFrameStyle(QFrame::Panel|QFrame::Raised);
    setMinimumSize(300, 200);

    d_grid.setPen(QPen(black, 0, DotLine));
    
    d_grid.setXDiv(d_sdiv[xBottom]);
    d_grid.setYDiv(d_sdiv[yLeft]);
    d_grid.enableXMin(FALSE);
    d_grid.enableYMin(FALSE);
    d_grid.setXAxis(xBottom);
    d_grid.setYAxis(yLeft);
    
    d_curves = new QwtCurveDict;
    d_markers = new QwtMarkerDict;

    d_frmPlot->setCursor(crossCursor);
    connect(d_frmPlot, SIGNAL(pixmapResized()), SLOT(updatePixmap()));
    connect(d_legend, SIGNAL(clicked(int)), SLOT(lgdClicked(int)));
    
}

//! Destructor
QwtPlot::~QwtPlot()
{
    int i;

    delete d_curves;
    delete d_markers;

    delete d_frmPlot;
    for (i=0;i<axisCnt;i++)
       delete d_scale[i];
    delete d_lblTitle;
    delete d_legend;
}


void QwtPlot::autoRefresh()
{
     if (d_autoReplot) replot();
}

/*!
  \brief Enables or disables outline drawing.
  When the outline feature is enabled, a shape will be drawn
  in the plotting region  when the user presses
  or drags the mouse. It can be used to implement crosshairs,
  mark a selected region, etc.
  \param tf \c TRUE (enabled) or \c FALSE (disabled)
  \warning An outline style has to be specified.
  \sa setOutlineStyle()
*/
void QwtPlot::enableOutline(bool tf)
{
    d_frmPlot->enableOutline(tf);
}

//! Initialize axes
void QwtPlot::initAxes()
{
    int i;

    QFont fscl("Helvetica", 10);
    QFont fttl("Helvetica", 12, QFont::Bold);
 
    
    for(i=0;i<axisCnt;i++)
    {
	d_scale[i]->setFont(fscl);
	d_scale[i]->setTitleFont(fttl);
    }

    d_axisEnabled[yLeft] = TRUE;
    d_axisEnabled[yRight] = FALSE;
    d_axisEnabled[xBottom] = TRUE;
    d_axisEnabled[xTop] = FALSE;

    for (i=0;i<axisCnt;i++)
    {
	d_as[i].adjust(0.0,1000.0,TRUE);
	d_sdiv[i] = d_as[i].scaleDiv();
	d_map[i].setDblRange(d_sdiv[i].lBound(), d_sdiv[i].hBound(), d_sdiv[i].logScale());
	d_scale[i]->setScale(d_sdiv[i]);
    }

}

/*!
  \brief Set or reset the autoReplot option
  If the autoReplot option is set, the plot will be
  updated implicitly by manipulating member functions.
  Since this may be time-consuming, it is recommended
  to leave this option switched off and call replot()
  explicitly if necessary.

  The autoReplot option is set to FALSE by default, which
  means that the user has to call replot() in order to make
  changes visible.
  \param tf \c TRUE or \c FALSE. Defaults to \c TRUE.
  \sa replot()
*/
void QwtPlot::setAutoReplot(bool tf)
{
    d_autoReplot = tf;
}

/*!
  \brief Change the plot's title
  \param t new title
*/
void QwtPlot::setTitle(const char *t)
{
    QString txt(d_lblTitle->text());
    QString ntxt(t);
    
    d_lblTitle->setText(t);

    if (txt.isEmpty() && (!ntxt.isEmpty()))
    {
	d_titleEnabled = TRUE;
	resize(size());
    }
    else if (ntxt.isEmpty() && (!txt.isEmpty()))
    {
	d_titleEnabled = FALSE;
	resize(size());
    }
	
}

/*!
  \return the plot's title
*/

QString QwtPlot::title() const
{
	if ( d_titleEnabled && d_lblTitle )
        return d_lblTitle->text();

    return QString::null;
}


/*!
  \brief Change the title font
  \param f new title font
*/
void QwtPlot::setTitleFont(const QFont &f)
{
    d_lblTitle->setFont(f);
    this->resize(this->size());
}

/*!
  \return the plot's title font
*/
QFont QwtPlot::titleFont() const
{
	return d_lblTitle->font();
}

/*!
  \brief Redraw the plot
  If the autoReplot option is not set (which is the default)
  or if any curves are attached to raw data, the plot has to
  be refreshed explicitly in order to make changes visible.
  \sa setAutoReplot()
*/
void QwtPlot::replot()
{
    int ar = d_autoReplot;
    d_autoReplot = FALSE;
    updateAxes();
    d_frmPlot->clearPixmap();
    updatePixmap();
    d_frmPlot->updatePixmap();
    d_autoReplot = ar;
}

void QwtPlot::resizeEvent(QResizeEvent *e)
{
    QFrame::resizeEvent(e);
    updatePlot();
}

void QwtPlot::updatePlot()
{
    QRect r, rTitle, rTop, rBottom, rLeft, rRight;
    QwtRect rPlot, rLegend;
    
    int hDist, vDist, h;
    

    //
    //  Determine the minimum distances of the
    //	scale origins from the border of frmPlot.
    //
    vDist = hDist = d_frmPlot->frameWidth() + 2;

    rPlot = this->contentsRect();
    
    if (d_axisEnabled[xTop]) 
        hDist = qwtMax(hDist, d_scale[xTop]->minBorderDist());
    if (d_axisEnabled[xBottom]) 
	hDist = qwtMax(hDist, d_scale[xBottom]->minBorderDist());
    
    if (d_axisEnabled[yLeft])
       vDist = qwtMax(vDist, d_scale[yLeft]->minBorderDist());
    if (d_axisEnabled[yRight])
       vDist = qwtMax(vDist, d_scale[yRight]->minBorderDist());


    //
    // Determine the rectangles of the visible elements
    //
    if (d_legendEnabled)
    {
	switch(d_legendPos)
	{
	case Qwt::Top:
	    d_legend->setMaxCols((rPlot.width() - 2 * d_margin - ScrBarWidth)
				 / d_legend->colWidth());
	    h = qwtMin(d_legend->sizeHint().height(), rPlot.height() / 5);
	    rLegend = rPlot.cutTop(h, d_margin);
	    rLegend.cutMargin(d_margin, d_margin,0,0);
	    break;
        case Qwt::Right:
	    d_legend->setMaxCols(1);
	    h = qwtMin(d_legend->sizeHint().width(), rPlot.width() / 5);
	    rLegend = rPlot.cutRight(h, 0, d_margin);
	    break;
	case Qwt::Left:
	    d_legend->setMaxCols(1);
	    h = qwtMin(d_legend->sizeHint().width(), rPlot.width() / 5);
	    rLegend = rPlot.cutLeft(h, d_margin);
	    rLegend.cutMargin(0,0,d_margin, d_margin);
	    break;
	default:
	    d_legend->setMaxCols( ( rPlot.width() - 2 * d_margin - ScrBarWidth )
				 / d_legend->colWidth() );
	    h = qwtMin(d_legend->sizeHint().height(), rPlot.height() / 5);
	    rLegend = rPlot.cutBottom(h, 0, d_margin);
	    rLegend.cutMargin(d_margin, d_margin,0,0);
	    break;
	}
    }
    
    if (d_titleEnabled)
       rTitle = rPlot.cutTop(d_lblTitle->sizeHint().height(), d_margin,
			     d_margin);
    else
       rTitle = rPlot.cutTop(d_margin);

    if (d_axisEnabled[xTop])
       rTop = rPlot.cutTop(d_scale[xTop]->sizeHint().height());

    if (d_axisEnabled[xBottom])
       rBottom = rPlot.cutBottom(d_scale[xBottom]->sizeHint().height(), 0, d_margin);
    else
       rPlot.cutBottom(qwtMax(d_margin, vDist));
    
    if (d_axisEnabled[yLeft])
       rLeft = rPlot.cutLeft(d_scale[yLeft]->sizeHint().width());
    else
       rLeft = rPlot.cutLeft(qwtMax(d_margin, hDist));
    
    if (d_axisEnabled[yRight])
       rRight = rPlot.cutRight(d_scale[yRight]->sizeHint().width());
    else
       rRight = rPlot.cutRight(qwtMax(d_margin, hDist)) ;


    //
    // Center scales and title
    //
    rTop.setLeft(rPlot.x() + vDist - hDist);
    rTop.setRight(rPlot.right() + hDist - vDist);
    rBottom.setLeft(rTop.left());
    rBottom.setRight(rTop.right());

    rTitle.setWidth(rPlot.width() + 2 * qwtMin(rLeft.width(), rRight.width()));
    rTitle.moveBy( rPlot.center().x() - rTitle.center().x(), 0 );

    //
    // Correct legend position
    //
    if (d_legendEnabled)
    {
	switch(d_legendPos)
	{

	case Qwt::Right:
	case Qwt::Left:

	    if (rLegend.height() > d_legend->sizeHint().height() + ScrBarWidth)
	    {
		// adjust the height; prevent scrollbars where possible
		if (rLegend.width() >= d_legend->sizeHint().width()) // has no hor. scrollbar
		   rLegend.setHeight(d_legend->sizeHint().height());
		else
		   rLegend.setHeight(d_legend->sizeHint().height() + ScrBarWidth);
		
		// put it at same the y position as rPlot if useful; center otherwise 
		if (rLegend.height() <= rPlot.height())
		    rLegend.moveBy(0, rPlot.y() - rLegend.y());
		else
		    rLegend.moveBy(0, contentsRect().center().y() - rLegend.center().y());
	    }
	    break;

	default:

	    if (rLegend.width() > (d_legend->sizeHint().width() + ScrBarWidth))
	    {
		// adjust the width
		if (rLegend.height() >= d_legend->sizeHint().height()) // has no vert. scrollbar
		   rLegend.setWidth(d_legend->sizeHint().width());
		else
		   rLegend.setWidth(d_legend->sizeHint().width() + ScrBarWidth);

		// put it in the center 
		if (rLegend.width() <= (rPlot.width() + 2 * qwtMin(rLeft.width(), rRight.width())))
		    rLegend.moveBy(rPlot.center().x() - rLegend.center().x(), 0);
		else
		    rLegend.moveBy(contentsRect().center().x() - rLegend.center().x(), 0);
	    }
	    break;
	}
    }
    
    
    //
    // resize and show the visible widgets 
    //
    if (d_titleEnabled)
    {
	d_lblTitle->setGeometry(rTitle);
	if (!d_lblTitle->isVisible()) d_lblTitle->show();
    }
    else
       d_lblTitle->hide();
    
    if (d_axisEnabled[yLeft])
    {
	d_scale[yLeft]->setBorderDist(vDist, vDist);
	d_scale[yLeft]->setGeometry(rLeft);
	if (!d_scale[yLeft]->isVisible()) d_scale[yLeft]->show();
    }
    else
       d_scale[yLeft]->hide();
 
    if (d_axisEnabled[yRight])
    {
	d_scale[yRight]->setBorderDist(vDist, vDist);
	d_scale[yRight]->setGeometry(rRight);
	if (!d_scale[yRight]->isVisible()) d_scale[yRight]->show();
    }
    else
       d_scale[yRight]->hide();
    
    if (d_axisEnabled[xTop])
    {
	d_scale[xTop]->setBorderDist(hDist, hDist);
	d_scale[xTop]->setGeometry(rTop);
	if (!d_scale[xTop]->isVisible()) d_scale[xTop]->show();
    }
    else
       d_scale[xTop]->hide();

    if (d_axisEnabled[xBottom])
    {
	d_scale[xBottom]->setBorderDist(hDist, hDist);
	d_scale[xBottom]->setGeometry(rBottom);
	if (!d_scale[xBottom]->isVisible()) d_scale[xBottom]->show();
    }
    else
       d_scale[xBottom]->hide();


    if (d_legendEnabled)
    {
	d_legend->setGeometry(rLegend);
	d_legend->show();
    }
    else
       d_legend->hide();
    
    
    //
    // update the axes.
    //
    updateAxes();
    
    //
    // adjust the maps in order to fit the pixmap rectangle
    //
    r.setRect(0,0,rPlot.width() - 2*d_frmPlot->frameWidth(),
	      rPlot.height() - 2*d_frmPlot->frameWidth());
    vDist = qwtMax(0,vDist - d_frmPlot->frameWidth());
    d_map[xBottom].setIntRange(vDist, r.right() - vDist);
    d_map[xTop].setIntRange(vDist, r.right() - vDist);
    d_map[yLeft].setIntRange(r.bottom() - vDist, vDist);
    d_map[yRight].setIntRange(r.bottom() - vDist, vDist);

    d_frmPlot->setGeometry(rPlot);

}



//! Rebuild the scales and maps
void QwtPlot::updateAxes()
{
    int i;
    QwtPlotCurve *c;

    QIntDictIterator<QwtPlotCurve> itc(*d_curves);

    //
    //  Adjust autoscalers
    //
    
    for (i=0; i<axisCnt; i++)
       d_as[i].reset();

    itc.toFirst();
    while((c = itc.current()))
    {
	if (c->dataSize() > 0)	// don't scale curves with no data
	{			//  (minXValue() et al. would return 0.0)
	    d_as[c->xAxis()].adjust(c->minXValue(), c->maxXValue());
	    d_as[c->yAxis()].adjust(c->minYValue(), c->maxYValue());
	}
	++itc;
    }

    //
    // Adjust scales
    //
    for (i=0; i<axisCnt; i++)
    {
	d_sdiv[i] = d_as[i].scaleDiv();
	d_map[i].setDblRange(d_sdiv[i].lBound(), d_sdiv[i].hBound(), d_sdiv[i].logScale());
	d_scale[i]->setScale(d_sdiv[i]);
    }

    d_grid.setXDiv(d_sdiv[d_grid.xAxis()]);
    d_grid.setYDiv(d_sdiv[d_grid.yAxis()]);
    
}


//! Redraw grid, curves, and markers
void QwtPlot::updatePixmap()
{

    QPainter p;
    QwtPlotCurve *curCurve;
    QwtPlotMarker *curMarker;
    QRect pixrect = d_frmPlot->pixmap()->rect();

    if (pixrect.isEmpty()) return;
    
    p.begin(d_frmPlot->pixmap());
    p.setClipping(TRUE);
    p.setClipRect(d_frmPlot->pixmap()->rect());

    //
    // draw grid
    //
    
    d_grid.draw(&p, pixrect, d_map[d_grid.xAxis()], d_map[d_grid.yAxis()]); 

     
    //
    //  draw curves
    //
    QIntDictIterator<QwtPlotCurve> itc(*d_curves);
    itc.toFirst();
    while((curCurve = itc.current()))
    {
	curCurve->draw(&p, d_map[curCurve->xAxis()], d_map[curCurve->yAxis()]);
	++itc;
    }
    
    //
    // draw markers
    //
    QIntDictIterator<QwtPlotMarker> itm(*d_markers);
    itm.toFirst();
    while((curMarker = itm.current()))
    {
    	curMarker->draw(&p,
			d_map[curMarker->xAxis()].transform(curMarker->xValue()),
			d_map[curMarker->yAxis()].transform(curMarker->yValue()),
			pixrect);
	++itm;
    }

   p.end();

}

/*!
  \brief Change the background of the plotting area
  \param c new background color
*/
void QwtPlot::setPlotBackground(const QColor &c)
{
    d_frmPlot->setPixmapBackground(c);
    autoRefresh();
}

/*!
  \brief Change the border width of the plotting area
  \param w new border width
*/
void QwtPlot::setPlotBorder(int w)
{
    w = qwtLim(w, 0, 10);
    d_plotBorder = w;
    d_frmPlot->setLineWidth(w);
    this->resize(this->size());
}


/*!
  \brief Specify the style of the outline

  The outline style determines which kind of shape is drawn
  in the plotting region when the user presses a mouse button
  or drags the mouse. Valid Styles are:
  \param os Outline Style. Valid values are: \c Qwt::HLine, \c Qwt::VLine,
            \c Qwt::Cross, \c Qwt::Rect, \c Qwt::Ellipse
  <dl>
  <dt>Qwt::Cros
  <dd>Cross hairs are drawn across the plotting area
      when the user presses a mouse button. The lines
      intersect at the point where the mouse was pressed
      and move with the mouse pointer.
  <dt>Qwt::HLine, Qwt::VLine
  <dd>A horizontal or vertical line appears when
      the user presses a mouse button. This is useful
      for moving line markers.
  <dt>Qwt::Rect
  <dd>A rectangle is displayed when the user drags
      the mouse. One corner is fixed at the point where
      the mouse was pressed, and the opposite corner moves
      with the mouse pointer. This can be used for selecting
      regions.
  <dt>Qwt::Ellipse
  <dd>Similar to Qwt::Rect, but with an ellipse inside
      a bounding rectangle.
  </dl>
  \sa enableOutline()
*/
void QwtPlot::setOutlineStyle(Qwt::Shape os)
{
    d_frmPlot->setOutlineStyle(os);
}



/*!
  \brief Specify a pen for the outline
  \param pn new pen
*/
void QwtPlot::setOutlinePen(const QPen &pn)
{
    d_frmPlot->setOutlinePen(pn);
}




/*!
  \brief Print the plot to a QPrinter
  This function prints the contents of a QwtPlot instance to
  QPrinter object. The printed output is
  independent of the widget displayed on the screen.

  \param prThe printer
  \param cfilter color filter for curves and markers
  \warning
  \li The background of the plot is not printed.
  \li Scales and titles are always printed in black color.
  \li major gridlines are printed in dark gray; minor gridlines are
      printed in gray.
  \li Curves, markers, symbols, etc. are printed in their screen colors
      if no color filter is specified. A color filter may be useful if
      curves have bright colors (which would be hardly visible), or
      if a grayscale output is desired.
  \li Font sizes and line widths are not scaled.
  \li The size of the printed plot is determined automatically,
      dependent on the page size.
  \par Example
  \verbatim
myPlot->print(printer);                  // screen colors
myPlot->print(printer, QwtFltrDim(200)); // colors darkened
myPlot->print(printer, QwtFltrInv());    // colors inverted
  \endverbatim
  \bugs
  \li The legend is not printed.
  \li QPrinter has some bugs in Qt 1.2 (see Qt manual on QPrinter).
  \sa QwtColorFilter
*/
void QwtPlot::print(QPrinter &pr, const QwtColorFilter &cfilter)
{
    QRect r, rPlot;
    QPen p1, p2,p3;			// buffers for some pens
    
    int th,w,i;

    int arf = d_autoReplot;
    d_autoReplot = FALSE;
    
    QwtPlotCurve *curCurve;
    QwtPlotMarker *curMarker;
    
    QPaintDeviceMetrics mpr(&pr);
    QwtScaleDraw sd[axisCnt];
    QwtDiMap map[axisCnt];
    QwtCurve cpCurve;
    QwtMarker cpMarker;
    QwtSymbol cpSym;
    QwtGrid cpGrid;
    QPen cpPen;
    QBrush cpBrush;

    { // Scoping brackets to make docName and pos local variables
      // d_lblTitle may be multi-line, but PostScript cannot handle this.
      // Thus we replace each "\n" */
      QString docName = d_lblTitle->text();
      docName.replace (QRegExp (QString::fromLatin1 ("\n")), tr (" -- "));
      pr.setDocName (docName);
    }

    pr.setCreator("QwtPlot");
    int printMargin;

    int ttpos = 0, btpos = 0;
     
    QPainter p;

    printMargin = qwtMin(mpr.width(), mpr.height()) / 10;
    

    r = QRect(printMargin, printMargin,
		 mpr.width() - 2 * printMargin,
		 mpr.height()- 2 * printMargin );

    if (r.isNull()) return;

    
    rPlot = r;

    rPlot.setRect(r.left(),
		 r.top(),
		 r.width(), qwtMin(r.height(), int(0.66 * double(r.width()))));

    sd[yLeft].setGeometry(rPlot.left(), rPlot.top(),
			  rPlot.height(),QwtScaleDraw::Left);
    sd[yRight].setGeometry(rPlot.left(), rPlot.top(),
			   rPlot.height(),QwtScaleDraw::Right);
    sd[xTop].setGeometry(rPlot.left(), rPlot.top(),
			 rPlot.height(),QwtScaleDraw::Top);
    sd[xBottom].setGeometry(rPlot.left(), rPlot.top(),
			    rPlot.height(),QwtScaleDraw::Bottom);

    for ( int axis = 0; axis < axisCnt; axis++ )
	{
	sd[axis].setScale(d_as[axis].scaleDiv());

	// get the axis label format and initialize the scale draw

	char f;
	int prec;
	axisLabelFormat(axis, f, prec);
	sd[axis].setLabelFormat(f, prec);
	}

    if ((mpr.width() == 0) || (mpr.height() == 0) || this->size().isNull())
       return;
    
    p.begin(&pr);

    p.setFont(d_lblTitle->font());
    p.setPen(black);


    if (d_titleEnabled)
    {
      /* Old code (0.2h)
	p.drawText(rPlot.x() + ( rPlot.width() -
			    p.fontMetrics().width(d_lblTitle->text())) / 2,
		   r.y() + p.fontMetrics().ascent() + 1,
		   d_lblTitle->text());
	rPlot.setTop(rPlot.top() + p.fontMetrics().height() * 2);
      */
        // New code (Uwe Rathmann), may need testing...
        QRect rect;
	p.drawText( rPlot, AlignTop | AlignHCenter | WordBreak,
		    d_lblTitle->text(), -1, &rect);
	rPlot.setTop(rPlot.top() + rect.height());
	// End of new code
    }
    th = rPlot.top();

    if (d_axisEnabled[yLeft])
    {
	p.setFont(d_scale[yLeft]->titleFont());
	w = p.fontMetrics().height() * 2;
	p.setFont(d_scale[yLeft]->font());
	w += sd[yLeft].maxWidth(&p, FALSE);
	rPlot.setLeft(rPlot.left() + w);
    }
    
    if (d_axisEnabled[yRight])
    {
	p.setFont(d_scale[yRight]->titleFont());
	w = p.fontMetrics().height() * 2;
	p.setFont(d_scale[yRight]->font());
	w += sd[yRight].maxWidth(&p, FALSE);
	rPlot.setRight(rPlot.right() - w);
    }
    
    if (d_axisEnabled[xTop])
    {
	p.setFont(d_scale[xTop]->titleFont());
	w = p.fontMetrics().height() * 2;
	ttpos = rPlot.top() + p.fontMetrics().height();
	p.setFont(d_scale[xTop]->font());
	w += sd[xTop].maxHeight(&p);
	rPlot.setTop(rPlot.top() + w);
    }

    if (d_axisEnabled[xBottom])
    {
	p.setFont(d_scale[xBottom]->titleFont());
	w = p.fontMetrics().height() * 2;
	btpos = rPlot.bottom() - p.fontMetrics().descent() - 1;
	p.setFont(d_scale[xBottom]->font());
	w += sd[xBottom].maxHeight(&p);
	rPlot.setBottom(rPlot.bottom() - w);
    }

    //
    // build maps
    //
    for (i=0;i<axisCnt;i++)
	map[i].setDblRange(d_sdiv[i].lBound(), d_sdiv[i].hBound(), d_sdiv[i].logScale());

    map[yLeft].setIntRange(rPlot.bottom(), rPlot.top());
    map[yRight].setIntRange(rPlot.bottom(), rPlot.top());
    map[xTop].setIntRange(rPlot.left(), rPlot.right());
    map[xBottom].setIntRange(rPlot.left(), rPlot.right());

    p.setPen(black);
    p.setBrush(NoBrush);
//    p.save();

    //
    // draw scales
    //
    if (d_axisEnabled[yLeft])
    {
	p.setFont(d_scale[yLeft]->font());
	sd[yLeft].setGeometry(rPlot.left(), rPlot.top(), rPlot.height(),QwtScaleDraw::Left);
	sd[yLeft].draw(&p);
	p.setFont(d_scale[yLeft]->titleFont());
	p.translate(r.left(), rPlot.bottom());
	p.rotate(-90.0);
	p.drawText(  ( rPlot.height()
		      - p.fontMetrics().width(d_scale[yLeft]->title()) )  / 2,
		   p.fontMetrics().ascent() + 1,
		   d_scale[yLeft]->title());
	p.resetXForm();
	
    }
    
    if (d_axisEnabled[yRight])
    {
	p.setFont(d_scale[yRight]->font());
	sd[yRight].setGeometry(rPlot.right(), rPlot.top(), rPlot.height(), QwtScaleDraw::Right);
	sd[yRight].draw(&p);
	
	p.setFont(d_scale[yRight]->titleFont());
	p.translate(r.right(), rPlot.top());
	p.rotate(90.0);
	p.drawText(  ( rPlot.height()
		      - p.fontMetrics().width(d_scale[yRight]->title()) ) / 2,
		   p.fontMetrics().ascent() + 1,
		   d_scale[yRight]->title());
	p.resetXForm();
    }

    if (d_axisEnabled[xTop])
    {
	p.setFont(d_scale[xTop]->font());
	sd[xTop].setGeometry(rPlot.left(), rPlot.top(), rPlot.width(),
			     QwtScaleDraw::Top);
	sd[xTop].draw(&p);

	p.setFont(d_scale[xTop]->titleFont());
	p.drawText( rPlot.left()
		   + ( rPlot.width() 
		      - p.fontMetrics().width(d_scale[xTop]->title()) ) / 2,
		   ttpos,
		   d_scale[xTop]->title());
    }

    if (d_axisEnabled[xBottom])
    {
	p.setFont(d_scale[xBottom]->font());
	sd[xBottom].setGeometry(rPlot.left(), rPlot.bottom(),
				rPlot.width(), QwtScaleDraw::Bottom);
	sd[xBottom].draw(&p);
	p.setFont(d_scale[xTop]->titleFont());
	p.drawText( rPlot.left()
		   + ( rPlot.width()
		      - p.fontMetrics().width(d_scale[xBottom]->title())) / 2,
		   btpos,
		   d_scale[xBottom]->title());
    }

//    p.restore();
//    p.save();

    //
    // draw grid
    //
    cpGrid = d_grid;
    cpPen = cpGrid.majPen();
    cpPen.setColor(darkGray);
    cpGrid.setMajPen(cpPen);
    cpPen = cpGrid.minPen();
    cpPen.setColor(gray);
    cpGrid.setMinPen(cpPen);
    
    cpGrid.draw(&p, rPlot, map[d_grid.xAxis()], map[d_grid.yAxis()]); 
    
//    p.restore();

    //
    //  draw curves
    //
    
    p.setClipping(TRUE);
    p.setClipRect(rPlot.x(), rPlot.y(), rPlot.width() - 1, rPlot.height() - 1);

    QIntDictIterator<QwtPlotCurve> itc(*d_curves);
    itc.toFirst();
    while((curCurve = itc.current()))
    {
	p.save();
	cpCurve = *curCurve;
	cpPen = cpCurve.pen();
	cpPen.setColor(cfilter(cpPen.color()));
	cpCurve.setPen(cpPen);

    QwtSymbol s = cpCurve.symbol();
    cpPen = s.pen();
    cpPen.setColor(cfilter(cpPen.color()));
    s.setPen(cpPen);
    cpCurve.setSymbol(s);

	cpCurve.draw(&p, map[curCurve->xAxis()], map[curCurve->yAxis()]);
	++itc;
	p.restore();
    }
    
    //
    // draw markers
    //
    QIntDictIterator<QwtPlotMarker> itm(*d_markers);
    itm.toFirst();
    while((curMarker = itm.current()))
    {
	p.save();
	cpMarker = *curMarker;

	cpPen = cpMarker.linePen();
	cpPen.setColor(cfilter(cpPen.color()));
	cpMarker.setLinePen(cpPen);

	cpPen = cpMarker.labelPen();
	cpPen.setColor(cfilter(cpPen.color()));
	cpMarker.setLabelPen(cpPen);
	
	cpSym = cpMarker.symbol();
	cpPen = cpSym.pen();
	cpPen.setColor(cfilter(cpPen.color()));
	cpSym.setPen(cpPen);

	cpBrush = cpSym.brush();
	cpBrush.setColor(cfilter(cpBrush.color()));
	cpSym.setBrush(cpBrush);
	cpMarker.setSymbol(cpSym);
	
    	cpMarker.draw(&p,
			map[curMarker->xAxis()].transform(curMarker->xValue()),
			map[curMarker->yAxis()].transform(curMarker->yValue()),
			rPlot);
	++itm;
	p.restore();
    }

    //draw bounding rectangle
    p.drawRect(rPlot.x(), rPlot.y(), rPlot.width() - 1, rPlot.height() - 1);
    
    p.end();

    d_autoReplot = arf;
}


/*!
  \return \c TRUE if the specified axis exists, otherwise \c FALSE
  \param axis axis index
 */
bool QwtPlot::axisValid(int axis)
{
    return ((axis >= QwtPlot::yLeft) && (axis < QwtPlot::axisCnt));
}


/*!
  \return the background color of the plotting area
*/
const QColor & QwtPlot::plotBackground() const
{
    return d_frmPlot->pixmapBackground();   
}

/*!
  \return \c TRUE if the outline feature is enabled
*/
bool QwtPlot::outlineEnabled() const
{
     return d_frmPlot->outlineEnabled();
}

/*!
  \return the pen used to draw outlines
*/
const QPen & QwtPlot::outlinePen() const
{
    return d_frmPlot->outlinePen();   
}

/*!
  \return the outline style
  \sa setOutlineStyle()
*/
Qwt::Shape QwtPlot::outlineStyle() const
{
    return d_frmPlot->outlineStyle();
}

/*!
  Called internally if a mouse button has been pressed.
*/
void QwtPlot::plotPressed(QMouseEvent *e)
{
    emit plotMousePressed(*e);  
}


/*!
  Called internally if a mouse button has been released.
  Emits a plotMouseReleased signal.
*/
void QwtPlot::plotReleased(QMouseEvent *e)
{
    emit plotMouseReleased(*e);
}


/*!
  Called internally if a mouse button has been released.
  Emits a plotMouseMoved signal.
*/
void QwtPlot::plotMoved(QMouseEvent *e)
{
    emit plotMouseMoved(*e);
}

/*!
  Called internally when the legend has been clicked on.
  Emits a legendClicked() signal.
*/
void QwtPlot::lgdClicked(int index)
{
    emit legendClicked(d_legend->key(index));
}

//! Remove all curves and markers
void QwtPlot::clear()
{
    d_legend->clear();
    d_curves->clear();
    d_markers->clear();
}


//! Remove all curves
void QwtPlot::removeCurves()
{
    d_curves->clear();
    d_legend->clear();
    replot();
}

//! Remove all markers
void QwtPlot::removeMarkers()
{
    d_markers->clear();
    replot();
}

//------------------------------------------------------------
//.C	Legend Properties
//------------------------------------------------------------

/*!
  \brief Enable or disable the legend
  \param tf \c TRUE (enabled) or \c FALSE (disabled)
  \param key FIXME
  \sa QwtPlot::setLegendPos()
*/
void QwtPlot::enableLegend(bool tf, long key)
{
    QwtPlotCurve *curCurve;

    if (key == -1) {
	if (tf) { // append missing items to the list
	    QIntDictIterator<QwtPlotCurve> itc(*d_curves);
	    itc.toFirst();
	    while ((curCurve = itc.current())) {
		uint index = d_legend->findFirstKey(itc.currentKey());
		if (index == d_legend->itemCnt()) { 
		    d_legend->appendItem(curCurve->title(), curCurve->symbol(),
					 curCurve->pen(), itc.currentKey());
		    autoRefresh();
		}
		++itc;
	    }
	} else {
	    d_legend->clear();
	    autoRefresh();
	}
    } else {
	uint index = d_legend->findFirstKey(key);
	if (tf) {
	    if (index == d_legend->itemCnt()) {
		if ((curCurve = d_curves->find(key))) {
		    d_legend->appendItem(curCurve->title(), curCurve->symbol(),
					 curCurve->pen(), key);
		    autoRefresh();
		}
	    }
	} else {
	    if (index != d_legend->itemCnt()) {
		d_legend->removeItem(index);
		autoRefresh();
	    }
	}
    }
    d_legendEnabled = (bool) d_legend->itemCnt();
    updatePlot();
}

/*!
  \brief Specify the position of the legend within the widget
  \param pos The legend's position. Valid values are \c Qwt::Left,
           \c Qwt::Right, \c Qwt::Top, \c QwtBottom.
*/
void QwtPlot::setLegendPos(int pos)
{
    if (pos != d_legendPos)
    {
	switch(pos)
	{
	case Qwt::Top:
	case Qwt::Bottom:
	    break;
	    
	case Qwt::Left:
	case Qwt::Right:
	    d_legend->setMaxCols(1);
	    break;
	default:
	    pos = Qwt::Bottom;
	    break;
	}
	d_legendPos = pos;
	this->resize(this->size());
    }
}

/*!
  \brief Change the font of the legend items
  \param fnew font
*/
void QwtPlot::setLegendFont(const QFont &f)
{
    d_legend->setFont(f);
    if (d_legendEnabled) resize(this->size());
}

/*!
  \brief Change the legend's frame style
  \param stFrame Style. See Qt manual on QFrame.
*/
void QwtPlot::setLegendFrameStyle(int st)
{
    d_legend->setFrameStyle(st);
}



/*!
  \return the frame style of the legend
*/
int QwtPlot::legendFrameStyle() const
{
    return d_legend->frameStyle(); 
}

/*!
  \return the font of the legend items
*/
const QFont QwtPlot::legendFont() const
{
    return d_legend->font(); 
}

